[
  {'base_name': 'one_service',
   'service_type': 'cone',
   'access_url': 'http://vo.cool_archive_cone.org/vo/conesearch',
   },
  {'base_name': 'another_service',
   'service_type': 'cone',
   'access_url': 'http://www.another_archive.org/big_catalog/conesearch',
   }
]
