[
    {'base_name': 'SampleSIAService',
    'service_type': 'xcone',
    'adql': '',
    'access_url': 'https://sia.some_archive.org/'
    'mean.votable?flatten_response=false&sort_by=distance'
    '&ra={}&dec={}&radius={}'
    }
]
