[
    {'base_name': 'PanSTARRS',
     'service_type': 'xcone',
     'adql': '',
     'access_url': 'https://catalogs.mast.stsci.edu/api/v0.1/panstarrs/dr2/'
     'mean.votable?flatten_response=false&raw=false&sort_by=distance'
     '&ra={}&dec={}&radius={}'
     }
]
